@@ -71,37 +71,24 @@ typedef enum {
 } EmpathyCapabilities;
 
 GType empathy_contact_get_type (void) G_GNUC_CONST;
-EmpathyContact * empathy_contact_new (TpContact *tp_contact);
 EmpathyContact * empathy_contact_from_tpl_contact (TpAccount *account,
     TplEntity *tpl_contact);
-EmpathyContact * empathy_contact_new_for_log (TpAccount *account,
-    const gchar *id, const gchar *alias, gboolean is_user);
 TpContact * empathy_contact_get_tp_contact (EmpathyContact *contact);
 const gchar * empathy_contact_get_id (EmpathyContact *contact);
-void empathy_contact_set_id (EmpathyContact *contact, const gchar *id);
 const gchar * empathy_contact_get_alias (EmpathyContact *contact);
 void empathy_contact_set_alias (EmpathyContact *contact, const gchar *alias);
 void empathy_contact_change_group (EmpathyContact *contact, const gchar *group,
     gboolean is_member);
 EmpathyAvatar * empathy_contact_get_avatar (EmpathyContact *contact);
-void empathy_contact_set_avatar (EmpathyContact *contact,
-    EmpathyAvatar *avatar);
 TpAccount * empathy_contact_get_account (EmpathyContact *contact);
 FolksPersona * empathy_contact_get_persona (EmpathyContact *contact);
 void empathy_contact_set_persona (EmpathyContact *contact,
     FolksPersona *persona);
 TpConnection * empathy_contact_get_connection (EmpathyContact *contact);
 TpConnectionPresenceType empathy_contact_get_presence (EmpathyContact *contact);
-void empathy_contact_set_presence (EmpathyContact *contact,
-    TpConnectionPresenceType presence);
 const gchar * empathy_contact_get_presence_message (EmpathyContact *contact);
-void empathy_contact_set_presence_message (EmpathyContact *contact,
-    const gchar *message);
 guint empathy_contact_get_handle (EmpathyContact *contact);
-void empathy_contact_set_handle (EmpathyContact *contact, guint handle);
 EmpathyCapabilities empathy_contact_get_capabilities (EmpathyContact *contact);
-void empathy_contact_set_capabilities (EmpathyContact *contact,
-    EmpathyCapabilities capabilities);
 gboolean empathy_contact_is_user (EmpathyContact *contact);
 void empathy_contact_set_is_user (EmpathyContact *contact,
     gboolean is_user);
@@ -112,16 +99,24 @@ gboolean empathy_contact_can_voip_audio (EmpathyContact *contact);
 gboolean empathy_contact_can_voip_video (EmpathyContact *contact);
 gboolean empathy_contact_can_send_files (EmpathyContact *contact);
 gboolean empathy_contact_can_use_rfb_stream_tube (EmpathyContact *contact);
-gboolean empathy_contact_load_avatar_cache (EmpathyContact *contact,
-    const gchar *token);
 
+typedef enum {
+  EMPATHY_ACTION_CHAT,
+  EMPATHY_ACTION_AUDIO_CALL,
+  EMPATHY_ACTION_VIDEO_CALL,
+  EMPATHY_ACTION_VIEW_LOGS,
+  EMPATHY_ACTION_SEND_FILE,
+  EMPATHY_ACTION_SHARE_MY_DESKTOP,
+} EmpathyActionType;
+
+gboolean empathy_contact_can_do_action (EmpathyContact *self,
+    EmpathyActionType action_type);
 
 #define EMPATHY_TYPE_AVATAR (empathy_avatar_get_type ())
 GType empathy_avatar_get_type (void) G_GNUC_CONST;
 EmpathyAvatar * empathy_avatar_new (guchar *data,
     gsize len,
     gchar *format,
-    gchar *token,
     gchar *filename);
 EmpathyAvatar * empathy_avatar_ref (EmpathyAvatar *avatar);
 void empathy_avatar_unref (EmpathyAvatar *avatar);
@@ -134,6 +129,9 @@ gboolean empathy_contact_equal (gconstpointer contact1,
     gconstpointer contact2);
 
 EmpathyContact *empathy_contact_dup_from_tp_contact (TpContact *tp_contact);
+EmpathyContact * empathy_contact_dup_best_for_action (
+    FolksIndividual *individual,
+    EmpathyActionType action_type);
 
 G_END_DECLS
 
